# Written by Gage Condon
# Circuit Playground Lightsaber Code
# 10/07/2021

import board, time, digitalio, neopixel, busio, adafruit_lis3dh
from audiopwmio import PWMAudioOut as AudioOut
from audiocore import WaveFile
from adafruit_led_animation.animation.pulse import Pulse

#direct audio output to AUDIO port on board
audio = AudioOut(board.AUDIO)

#function that prepares a wave file to play later on
def prepare_sound(filename):
    wave_file = open("sounds/"+filename, "rb")
    wave = WaveFile(wave_file)
    return wave

#open sound files and prepare them for later
on_sound = prepare_sound("on.wav")
idle_sound = prepare_sound("idle.wav")
swing_sound = prepare_sound("swing.wav")
off_sound = prepare_sound("off.wav")

#set up light strip
light_strip_pin = board.A6
num_of_strip_lights = 30
light_strip = neopixel.NeoPixel(light_strip_pin, num_of_strip_lights, brightness = 0.5, auto_write=True)

#set up colors
RED = (255, 0, 0)
ORANGE = (255, 40, 0)
YELLOW = (255, 150, 0)
GREEN = (0, 255, 0)
BLUE = (0, 0, 255)
INDIGO = (63, 0, 255)
VIOLET = (127, 0 , 255)
WHITE = (255, 255, 255)
colors = [RED, ORANGE, YELLOW, GREEN, BLUE, INDIGO, VIOLET, WHITE]

#set up accelerometer
i2c = busio.I2C(board.ACCELEROMETER_SCL, board.ACCELEROMETER_SDA)
int1 = digitalio.DigitalInOut(board.ACCELEROMETER_INTERRUPT)
accelerometer = adafruit_lis3dh.LIS3DH_I2C(i2c, address=0x19, int1=int1)
accelerometer.range = adafruit_lis3dh.RANGE_8_G

#set up on/off button as on_button
on_button = digitalio.DigitalInOut(board.A3)
on_button.direction = digitalio.Direction.INPUT
on_button.pull = digitalio.Pull.UP

#set up color changeing button as color_button
color_button = digitalio.DigitalInOut(board.A4)
color_button.direction = digitalio.Direction.INPUT
color_button.pull = digitalio.Pull.UP

#variable to hold status of lightsaber
saber_on = False

#variable to hold the current color of the lightsaber
current_color_index = 7

#function to turn lightsaber on
def turn_saber_on():
    print("Lightsaber is turning on!")
    saber_on = True

    #play on sound
    audio.play(on_sound)
    while audio.playing:
        #turn on lights from bottom to top while on sound plays
        for i in range(0,15):
            light_strip[i] = WHITE
            light_strip[29-i] = WHITE

    #start playing idle sound
    audio.play(idle_sound, loop=True)

    return saber_on

#function to turn lightsaber off
def turn_saber_off():
    print("Lightsaber is turning off :(")
    saber_on = False

    #play on sound
    audio.play(off_sound)
    while audio.playing:
        #turn on lights from bottom to top while on sound plays
        for i in range(15,-1, -1):
            light_strip[i] = (0,0,0)
            light_strip[29-i] = (0,0,0)
    current_color_index = 7
    return saber_on, current_color_index

def change_color(current_color_index):
    #if the index is on the last one, reset it to 0. Otherwise, increment by 1
    if current_color_index==7:
        current_color_index=0
    else:
        current_color_index+=1

    #light up the lights on both sides from top to bottom
    for i in range(0,15):
        light_strip[i] = colors[current_color_index]
        light_strip[29-i] = colors[current_color_index]

    #return the updated color_index
    return current_color_index

def check_swing():
    x, y, z = accelerometer.acceleration
    accel_total = x * x + z * z
    if accel_total > 150:
        audio.play(swing_sound, loop=False)
        time.sleep(.8)
        print("swing detected!")
        audio.play(idle_sound, loop=True)

while True:

    #if on/off button pressed and lightsaber is not on...
    if not on_button.value and saber_on==False:
        #set saber_on to False and turn lightsaber on
        saber_on = turn_saber_on()

    #if on/off button pressed and lightsaber is on...
    elif not on_button.value and saber_on==True:
        saber_on, current_color_index = turn_saber_off()

    #if the lightsaber is on...
    if saber_on==True:

        #check for color_button press between idle sound loops
        if not color_button.value:
            current_color_index = change_color(current_color_index)

        #check for a swing
        check_swing()

    time.sleep(0.05)

